function [sample,wfm,par,sensortype]=InterpretTRC(sourcefile)
% function [sample,wfm,par]=InterpretTRC(sourcefile)

% Lars Hoff, HVE, Jul 2003
%                 Feb 2005: Modified for different sensor types

oldpath=cd;
if nargin<1, sourcefile= 0;     end

%-- Select and open raw data file ---
fprintf('\n')
fprintf('Finding raw data')
if isequal(sourcefile,0)
  [sourcefile, pathname] = uigetfile('*.trc', 'Select raw data file');
  if isequal(sourcefile,0) | isequal(pathname,0)
    disp('No file selected');
    return
  else
    cd(pathname);
  end  
end
fprintf('\n - ')
fprintf('Opening raw data file %s', sourcefile);%
[pathstr,filename,ext] = fileparts(sourcefile);
src=fopen(sourcefile,'r','ieee-be');  % LabView uses 'Big-Endian'

%--- Read header and parameters ---
fprintf('\n - ')
fprintf('Reading file header');
header    = fgetl(src);                   % Header line 
sample    = ReadParameters(src,'<EOH>');  % Sample data 

sensortype= floor(sscanf(sample.sensor{2}, 'MHP_%d')/100);
sample.sensortype=sensortype;
binheader = InterpretBinaryHeader(src,sensortype);   % Binary header

%--- Read measurement data ---
fprintf('\n - ')
fprintf('Reading traces');
switch sensortype
    case 0, [trace,N] = fread(src,[binheader.N,inf],'*int16');
    case 1, [trace,N] = fread(src,[binheader.N,inf],'*uint16');
end

%--- Waveform results ---
wfm = binheader;
wfm.trace= trace;
switch sensortype
    case 0, wfm.tracename = { 'Vdd' 'ECG' 'a1' 'a2' 'a3' 'a4' 'b1' 'b2' 'b3' 'b4' }';
            wfm.axname    = { 'Vdd' 'ECG' 'Ax' 'Az' 'Ax' 'Ay' 'Bx' 'Bz' 'Bx' 'By' }';
            axisorder = [1 4 2];    % xyz-axes mapped to sensor channels
            Nch       = 4;           % No. of channels per sensor
            ch= [ axisorder axisorder+Nch];
            mt= [1 2 2+ch];
            
    case 1, wfm.tracename = { 'Vdd' 'ECG' 'a1' 'a2' 'a3' 'b1' 'b2' 'b3'  }';
            wfm.axname    = { 'Vdd' 'ECG' 'Ax' 'Ay' 'Az' 'Bx' 'By' 'Bz' }';
            axisorder = [1 2 3 ];    % xyz-axes mapped to sensor channels
            Nch       = 3;           % No. of channels per sensor
            ch= [ axisorder axisorder+Nch];
            mt= [1 2 2+ch];
    otherwise, error('Unknown sensor type')
end
wfm= reorganize(wfm,mt);

fprintf('\n - ')
fprintf('Raw data file loaded');

%--- Close file and finish ---
fclose(src);

%--- Read extra parameter file --- 
parameterfile= sprintf('%s.dta',filename);
if exist(parameterfile)==2
fprintf('\n - ')
  fprintf('Reading extra parameter file %s',parameterfile);
  parid  = fopen(parameterfile,'r');
  header = fgetl(parid);                   % Header line 
  par    = ReadParameters(parid,'<EOP>');  % Sample data 
  fclose(parid);
else
  par=[];
end

fprintf('\n')
fprintf('Data ready');
fprintf('\n')

%cd(oldpath);
return


%=== FUNCTION READPARAMETERS =====================
function [result]=ReadParameters(src,endcode)
result= [];
eop=0;
while not(eop)
  ln= fgetl(src);
  eop=strcmpi(ln,endcode);
  if not(eop)
    [name,value]=InterpretLine(ln);
    result= setfield(result,name,value);
  end
end


%=== FUNCTION INTERPRETLINE ======================
function [name,value]=InterpretLine(ln)
k=find(ln==' ');
name  = ln(     1: k(1)-1 );
type  = ln(k(1)+1: k(2)-1 );
value = ln(k(2)+1:  end   );
switch(type)  
  % String value
  case 's', value=setstr(value);  
            k2=find(value==';');
            if not (isempty(k2))
              k1=1+[0 k2(1:end-1)];
              for k=1:length(k2)
                cvalue{k}=value(k1(k):k2(k)-1);
              end	   
              value=cvalue;
            end
            
  % Numeric value  
  case 'n',  value=str2num(value);
end

return

%=== FUNCTION H=INTERPRETBINARYHEADER ============
function h=InterpretBinaryHeader(src,sensortype)
switch sensortype
    case 0,
        N=fread(src,1,'int32');
        h.N              = fread(src,1,  'uint32');
        h.chlength       = fread(src,h.N,'uint32');
        h.channel        = interpretstring(src,h.chlength);
        h.Vup            = fread(src,h.N,'float32' );
        h.Vlo            = fread(src,h.N,'float32' );
        h.range          = fread(src,h.N,'float32' );
        h.polarity       = fread(src,h.N,'uint16'  );
        h.gain           = fread(src,h.N,'float32' );
        h.coupling       = fread(src,h.N,'uint16'  );
        h.inputmode      = fread(src,h.N,'uint16'  );
        h.scalemultiplier= fread(src,h.N,'float32' );
        h.scaleoffset    = fread(src,h.N,'float32' );
        h.samplerate     = fread(src,1,  'float32' );
    case 1, 
        h.ver            = fread(src,1,  'float64');
        h.N              = fread(src,1,  'uint32');
        h.inputconfig    = fread(src,1,  'int32');
        h.Lscalingcoeff  = fread(src,2,  'int32');
        h.scalingcoeff   = fread(src,flipud(h.Lscalingcoeff)','float64' )';
        h.Lchannel       = fread(src,1,  'int32');
        h.channel        = interpretstring(src, h.Lchannel );
        h.samplemode     = fread(src,1,  'int32');
        h.samplerate     = fread(src,1,  'float64');
        h.Lverstring     = fread(src,1,  'int32');
        h.verstring      = interpretstring(src, h.Lverstring );
        
        h.scaleoffset    = h.scalingcoeff(:,1);
        h.scalemultiplier= h.scalingcoeff(:,2);

    otherwise, error('Binary header: Unknown sensor type')
end
return


%=== FUNCTION INTERPRETSTRING ==============================
function s=interpretstring(src,L)
for n=1:length(L)
  bt   = fread(src, L(n),'char' );  
  cs{n}= setstr(bt');
end
s=char(cs);
return

%--- Extract an organize waveform parameters ---
function wfmn= reorganize(wfm,mt)
  wfmn.description= 'Waveform data organized.';
  wfmfields=fieldnames(wfm);
  for k=1:length(wfmfields)     
      f=getfield(wfm,wfmfields{k});
      [M N]=size(f);
      if M==wfm.N,  wfmn=setfield(wfmn,wfmfields{k}, f(mt,:));
      else,         wfmn=setfield(wfmn,wfmfields{k},f);
      end
  end
return

