function [sample,wfmr,scaled]=PlotMicroheart(measdata)
% function [sample,wfmr,scaled]=PlotMicroheart(measdata)
% $Id: PlotMicroheart.m 96 2007-02-07 11:45:26Z lah $
% Calculate and plot results of Microheart measurement

% Lars Hoff, HVE, Nov 2004
%
% Dec 2005: Completely rewritten 
%
% Ver 4, Jan 2007: 
%   Load only .mat-files, including pressure data.
%   .trc-files must be converted

%--- Measurement file organization constants  ---
axisorder = [1 2 3 ];    % xyz-axes mapped to sensor channels
Nch       = 3;           % No. of channels per sensor
ch= [ axisorder axisorder+Nch];
mt= [1 2 2+ch];

%--- Find and load raw data file ---
[sample,wfm,par]= LoadDatafile(measdata);
sample=sample(1);  % If result is a combination of two or more files.


%--- Find curves to include ---
curve=measdata.curve;
for k=1:measdata.Ncurves    
    include(k)= any(measdata.curve(k).sensor==['AB']); % Curves with a sensor selected
    curve(k).include=include(k);
%     if include(k)
%         m0=3*(measdata.curve(k).sensor-'A');
%         switch lower(measdata.curve(k).ax(1))
%             case 'l',  mi= m0+ [1:3];
%             otherwise, mi= m0+ lower(measdata.curve(k).ax(1))-'x'+1;
%         end
%         curve(k).mi =mi;
%    end
end

%=== SCALE TRACES AND INTEGRATE ===================================
%--- Limit calculation interval --
wfmr= wfm;
wfmr.timelimits= FindInterval(measdata.interval,wfm.samplerate);
wfmr.trace = wfm.trace(:,wfmr.timelimits.n0+[0:wfmr.timelimits.Nn-1]);

%--- Scale ---
scaled= Trace2Acc(sample,wfmr);
scaled.ECG.delay= measdata.ECGdelay*1e-3;  % ECG-delay input is in ms

if isfield(sample, 'pressuresource') & isfield(scaled,'p')
    if strcmp(sample.pressuresource, 'Gateway')
        scaled.p.delay= measdata.ECGdelay*1e-3;  % Pressure curves delayed in patient monitor
        fprintf('\nGateway: Pressure curves delayed %3.1f ms\n',measdata.ECGdelay)
    else
        fprintf('\nPressure curves NOT delayed\n' )
    end    
end

fprintf('\n');    
if measdata.respfilter<0
    switch upper(measdata.AP(2))
        case 'P', measdata.respfilter = 1.0;
                  fprintf('Pig.  ');                  
        case 'H', measdata.respfilter = 0.6;
                  fprintf('Human. ');    
        otherwise, error('Animal/patient not identified');
    end
end

fprintf('Resp. filter %4.2f Hz \n', measdata.respfilter);
scaled.fcutoff= measdata.respfilter;       % Cut-off frequency
scaled.trace  = scaled.a;  % Default, changed below if appropriate

%=== CALCULATE CURVES ===================================
for k=1:measdata.Ncurves    
    if curve(k).include
        
        offset= 3*(upper(curve(1).sensor)-'A');    % Select 3 axes based on sensor
        y  = scaled.a.y(:,offset+[1:3]);         
        
        %--- Set projection ---
        dir= curve(k).ax;    % Projection direction
        Md = length(dir);
        if Md== 3            % Project if three components
            y  = Projection(y,dir);
            dirtext = sprintf(' %d', dir);
            calculatelength= 0;
        elseif Md==1         % One component means 'Length'
            calculatelength= 1;
            dirtext = sprintf('Length');
        else
            error('Unknown projection')
        end
        
        fs= 1/scaled.a.dt;     % 1/s  Sample rate
        fc= scaled.fcutoff;    % Hz   Respiration filter cutoff
        switch lower(curve(k).type(1:4))
            case 'acce',
            case 'filt',  y= MHintegrate(y, fs, fc, 0 );
            case 'velo',  y= MHintegrate(y, fs, fc, 1 );
            case 'posi',  y= MHintegrate(y, fs, fc, 2 );
            otherwise, error('Unknown motion parameter'), return
        end
        if calculatelength,  y = sqrt(sum(y.^2,2));  end

        curve(k).t0= scaled.a.t0;
        curve(k).dt= scaled.a.dt;
        curve(k).y= y;

        curve(k).name= sprintf('%s  [%s]  %s',  curve(k).sensor, dirtext, curve(k).type);
    end
end
if not(isfield(scaled,'p')), scaled.p=[]; end   
    
%--- Plot results ---
figure(measdata.figno)
PlotMHTraces(curve,scaled.ECG,sample,measdata,scaled.p,scaled.dpdt);

%--- Clean up and finish ---
fprintf('\n--- Finished --- \n')
return

