function scaled= Trace2Acc(sample,wfmr, ch)
% function scaled= Trace2Acc(sample,wfmr)
%
% Calculate scaled acceleration vectors 
% Input traces in bits from ADC.
% Returns organized traces in volts and m/s2

% Lars Hoff, HVE, Nov 2004
% Changes: Dec 2005 : Split trace and parameter input variable
%          Jan 2007 : Input variable definition changed

if nargin<3
    ch=[1:6]; % Default channel organization    
end

if not(isfield(wfmr, 'timelimits'))
    wfmr.timelimits.t0 = 0;
    wfmr.timelimits.dt = 1/wfmr.samplerate;
    wfmr.timelimits.n0 = 1;
    wfmr.timelimits.N =  max(size(wfmr.trace));
end

%--- Scaling and constants --------------------------------------------
t0  = wfmr.timelimits.t0;
dt  = wfmr.timelimits.dt;;
g    = 9.8;   % m/s2   Acceleration of gravitation
mm_Hg= 100;   % mm/Hg  Scaling of pressure catheters
[M N]= size(wfmr.trace);

scaled=[];

fprintf('\n')
fprintf('Scaling ... ')

%--- Voltage vector. Needed by all ---
fprintf('Voltages ... ')

for k=1:M
    v(:,k)= double(wfmr.trace(k,:))'*wfmr.scalemultiplier(k)+wfmr.scaleoffset(k);
end

%--- Supply voltage and its stability ---
Vdd = v(:,1);                    % V  Measured supply voltage
Vddm = median(Vdd);
Vddlim = Vddm*(1+0.05*[-1 1]);      % V  Supply voltage limits
Ns  = ceil(1.0*wfmr.samplerate);  %    Interval to check stability within
Ms  = floor(N/Ns);        
Vdds= reshape(Vdd(1:Ns*Ms),Ns,Ms);  
dV  = std(Vdds)./mean(Vdds);     %    Voltage stablilty
in  = (dV<2e-2) & (max(Vdds)<Vddlim(2)) & (min(Vdds)>Vddlim(1));
in  = repmat(in,Ns,1);
in  = uint8(in(:));         % Index vector: Include point or not
Ni  = length(in);           % End point
ni  = find(in);             % Indexes to include 
nex = setdiff([1:Ni]', ni); % Indexes to exclude 

v   = v(1:Ni,:);
Vdd= Vdd(1:Ni);

%=== EXPORT ACCELERATION VECTOR ============================
fprintf('Calibration ...')

%--- Read sensor calibration file ---
bias        = [];
sensitivity = [];
for k=1:length(sample.sensor)
    if not(strcmpi(sample.sensor{k},'no_sensor'))        
        cal{k}=ReadCalibration(sample.sensor{k});
        bias        = [ bias        cal{k}(1,:) ];
        sensitivity = [ sensitivity cal{k}(2,:) ];
    end
end
bias       = bias(ch);
sensitivity= sensitivity(ch);
Na = min([M-2 length(bias)]);  % No. of calibrated acceleration curves

if Na==0, error('No calibration data available');  end

%--- Calibrate acceleration values ---
fprintf('Acceleration ... ')
a = zeros(Ni,Na);
for k=1:Na
    Vb = Vdd(ni)*bias(k);
    a(ni,k) = (v(ni,k+2)-Vb)./(Vdd(ni)*sensitivity(k));
    a(nex,k)= mean(a(ni,k));   % Excluded points replaced by mean value of included
end

%--- Normalized ECG ---
ECG    = zeros(Ni,1);
ECG(ni)= v(ni,2);
ECG    = ECG/max(abs(ECG));

%--- Pressure curves ---
if M>8  % Pressure data fixed at columns 9 to 12
    ECGp    = zeros(Ni,1);
    ECGp(ni)= v(ni,9);
    ECGp = ECGp/max(abs(ECGp));
    ECG= [ECG ECGp];
    
    Mp = min(3,M-9);
    p      = zeros(Ni, Mp);
    p(ni,1:Mp)= v(ni,10+[0:Mp-1]);
    [Np Mp]= size(p);
    pm= mean(p);
    for k=1:Mp
        if pm(k)<0
            p(:,k)= -p(:,k);  % Invert if negative
        end
    end
    p=p*mm_Hg;
    
    %--- Pressure derivatives ---
    dpdt= zeros(size(p));
    pl = p(1:N-2 , :);
    ph = p(3:N   , :);
    dpdt(1,:)= (p(2,:)-p(1,:))/dt;
    dpdt(N,:)= (p(N,:)-p(N-1,:))/dt;
    dpdt(2:N-1,:)= (ph-pl)/(2*dt);
else
    p   =[];
    dpdt= [];
end


%=== ORGANIZE IN STRUCTURE ================================
scaled.bias        = bias;
scaled.sensitivity = sensitivity;
scaled.Na          = Na;
scaled.excluded    = find(not(in));

scaled.Vdd = BuildTrace(t0,dt,Vdd  );
scaled.ECG = BuildTrace(t0,dt,ECG  );
scaled.a   = BuildTrace(t0,dt,a*g  );
scaled.p   = BuildTrace(t0,dt,p    );
scaled.dpdt= BuildTrace(t0,dt,dpdt );

fprintf('Ready')

return
